/* Revision History
 *
 * $Id: table.cc,v 1.5 2002/09/17 09:38:24 gvacanti Exp $
 *
 */


/**************************************************************************/
/*                                                                        */
/*  FILENAME                                                              */
/*  ========                                                              */
/*  table.h                                                               */
/*                                                                        */
/*                                                                        */
/*  SUBSYSTEM                                                             */
/*  =========                                                             */
/*  XMM SAS - UTILITIES                                                   */
/*                                                                        */
/*                                                                        */
/*  TITLE                                                                 */
/*  =====                                                                 */
/*  Public header file for interface to TableT type.                      */
/*                                                                        */
/*                                                                        */
/*  AUTHORS                                                               */
/*  =======                                                               */
/*  Mark Thomas                                                           */
/*  CARA Software & Services Ltd                                          */
/*                                                                        */
/*                                                                        */
/*  CONSTRAINTS                                                           */
/*  ===========                                                           */
/*  All typedefs and functions given in this header file are for          */
/*  public use.                                                           */
/*                                                                        */
/*                                                                        */
/*  DESCRIPTION                                                           */
/*  ===========                                                           */
/*  Public Header file for interface to TableT type.                      */
/*                                                                        */
/*                                                                        */
/*  EXAMPLE                                                               */
/*  =======                                                               */
/*                                                                        */
/*                                                                        */
/*  #include "table.h"                                                    */
/*                                                                        */
/*                                                                        */
/*  main()                                                                */
/*  {                                                                     */
/*  	TableT		Table;                                            */
/*                                                                        */
/*                                                                        */
/*                                                                        */
/*  }                                                                     */
/*                                                                        */
/*                                                                        */
/**************************************************************************/


#include "table.h"

#include <string>
#include <iostream>


#include "Tracer.h"



TableT::TableT()
{

	PROC
	_current = _table.begin();

	ENDPROC

}


TableT::TableT( const TableT & thisTable )
{
	PROC

	_table = thisTable._table;
	_current = thisTable._current;

	ENDPROC

}


TableT &
TableT::operator=( const TableT & thisTable )
{
	PROC

	_table = thisTable._table;
	_current = thisTable._current;

	RETURN *this;

}


TableT::~TableT()
{
	PROC
	ENDPROC
}


Token *
TableT::Insert(
	const string &	itemName,
	const Token &	thisItem)
{

	PROC

	TableItem			tableItem;
	list<TableItem>::iterator	k;

	tableItem._name = itemName;
	tableItem._value = thisItem;

	k = _table.insert( _table.end(), tableItem );


	RETURN &( (*k)._value );

}


Token &
TableT::Fetch( const string & itemName ) const
{

	PROC

	TableItem				tableItem;
	list<TableItem>::const_iterator	k;
	bool					found = false;


	for( k = _table.begin(); k != _table.end(); k++ )
	{
		tableItem = *k;

		if( tableItem._name == itemName )
		{
			found = true;

			break;

		} // if
	} // for

	if( !found )
	{
		cerr << itemName << " was not found." << endl;

		exit( 1 );

	} // if

	RETURN (Token &)(*k)._value;


}


Token &
TableT::operator[]( const string & name )
{
	PROC
	if( IsFound( name ))
	{
		RETURN Fetch( name );
	}
	else
	{
		Token	token;
		Insert( name, token );
		RETURN Fetch( name );
	} // if
}


void
TableT::first()
{
	PROC
	_current = _table.begin();

	ENDPROC

}


Token &
TableT::value() const
{
	PROC
	RETURN (*_current)._value;

}


void
TableT::modify(
	const string &		itemName,
	const Token &		withThisValue )
{
	PROC

	TableItem			tableItem;
	list<TableItem>::iterator	k;
	bool				found = false;


	for( k = _table.begin(); k != _table.end(); k++ )
	{
		tableItem = *k;

		if( tableItem._name == itemName )
		{
			found = true;

			break;

		} //if
	} // for

	if( !found )
	{
		cerr << itemName << " was not found. " << endl;

		exit( 1 );

	} /* if */

	(*k)._value = withThisValue;

	ENDPROC

}


void
TableT::next()
{
	PROC

	++_current;

	ENDPROC

}


int
TableT::more()
{
	PROC
	RETURN _current != _table.end();

}


const char *
TableT::name() const
{
	PROC
	RETURN (*_current)._name.c_str();

}


void
TableT::append( const TableT & thisTable )
{
	PROC

	list<TableItem>::const_iterator	i;


	for( i = thisTable._table.begin(); i != thisTable._table.end(); ++i )
	{
		Insert( (*i)._name, (*i)._value );

	} // for

	ENDPROC

}


int
TableT::IsFound( const string & itemName ) const
{

	PROC

	TableItem			tableItem;
	list<TableItem>::const_iterator	k;
	bool				found = false;

	for( k = _table.begin(); k != _table.end(); k++ )
	{
		tableItem = *k;

		if( tableItem._name == itemName )
		{
			found = true;

			break;

		} // if
	} // while

	RETURN found;


}


ostream &
operator << ( ostream & toThisStream, const TableT & thisTable )
{
	PROC

	list<TableT::TableItem>::const_iterator	k;
	unsigned int				i = 0;

	for( k = (thisTable._table).begin(); k != (thisTable._table).end(); k++ )
	{
		toThisStream << '[' << i++ << ']' << (*k)._name << ' '
			<< (*k)._value << endl;

	} // for

	RETURN toThisStream;

}

